#!/usr/bin/env python3
"""
表情分析器模块
使用OpenRouter的Google Gemma-3-27B-IT模型分析图像中的人物表情
"""

import json
import base64
import requests
from typing import Dict, Any
from pathlib import Path


class EmotionAnalyzer:
    def __init__(self, api_key: str):
        """
        初始化表情分析器
        
        Args:
            api_key: OpenRouter API密钥
        """
        self.api_key = api_key
        self.base_url = "https://openrouter.ai/api/v1"
        self.model = "google/gemma-2-27b-it:free"
        
    def encode_image(self, image_path: str) -> str:
        """
        将图像文件编码为base64字符串
        
        Args:
            image_path: 图像文件路径
            
        Returns:
            base64编码的图像字符串
        """
        try:
            with open(image_path, "rb") as image_file:
                return base64.b64encode(image_file.read()).decode('utf-8')
        except Exception as e:
            raise Exception(f"无法读取图像文件 {image_path}: {str(e)}")
    
    def analyze_emotion(self, image_path: str) -> Dict[str, Any]:
        """
        分析图像中人物的表情
        
        Args:
            image_path: 图像文件路径
            
        Returns:
            包含分析结果的字典
        """
        try:
            # 编码图像
            base64_image = self.encode_image(image_path)
            
            # 构建请求
            headers = {
                "Authorization": f"Bearer {self.api_key}",
                "Content-Type": "application/json"
            }
            
            # 构建提示词
            prompt = """请分析这张图片中人物的表情和情绪状态。请提供以下信息：

1. 主要表情类型（如：快乐、悲伤、愤怒、惊讶、恐惧、厌恶、中性等）
2. 表情强度（1-10分）
3. 具体的面部特征描述
4. 推测的情绪状态
5. 置信度评估

请以JSON格式返回结果，包含以下字段：
- emotion: 主要表情类型
- intensity: 表情强度(1-10)
- confidence: 置信度(0-1)
- facial_features: 面部特征描述
- emotional_state: 情绪状态描述
- additional_notes: 其他观察"""

            payload = {
                "model": self.model,
                "messages": [
                    {
                        "role": "user",
                        "content": [
                            {
                                "type": "text",
                                "text": prompt
                            },
                            {
                                "type": "image_url",
                                "image_url": {
                                    "url": f"data:image/jpeg;base64,{base64_image}"
                                }
                            }
                        ]
                    }
                ],
                "max_tokens": 1000,
                "temperature": 0.3
            }
            
            # 发送请求
            response = requests.post(
                f"{self.base_url}/chat/completions",
                headers=headers,
                json=payload,
                timeout=30
            )
            
            if response.status_code != 200:
                raise Exception(f"API请求失败: {response.status_code} - {response.text}")
            
            result = response.json()
            
            if 'choices' not in result or len(result['choices']) == 0:
                raise Exception("API返回结果格式错误")
            
            content = result['choices'][0]['message']['content']
            
            # 尝试解析JSON响应
            try:
                # 提取JSON部分
                if '```json' in content:
                    json_start = content.find('```json') + 7
                    json_end = content.find('```', json_start)
                    json_content = content[json_start:json_end].strip()
                elif '{' in content and '}' in content:
                    json_start = content.find('{')
                    json_end = content.rfind('}') + 1
                    json_content = content[json_start:json_end]
                else:
                    json_content = content
                
                parsed_result = json.loads(json_content)
                
                return {
                    "success": True,
                    "image_path": image_path,
                    "analysis": parsed_result,
                    "raw_response": content
                }
                
            except json.JSONDecodeError:
                # 如果无法解析JSON，返回原始文本
                return {
                    "success": True,
                    "image_path": image_path,
                    "analysis": {
                        "emotion": "unknown",
                        "intensity": 0,
                        "confidence": 0,
                        "facial_features": content,
                        "emotional_state": "无法解析结构化数据",
                        "additional_notes": "返回原始分析文本"
                    },
                    "raw_response": content
                }
                
        except Exception as e:
            return {
                "success": False,
                "image_path": image_path,
                "error": str(e)
            }