#!/usr/bin/env python3
"""
表情分析 MCP 服务器
提供标准 MCP 接口用于图像表情分析
"""

import asyncio
import json
import os
import sys
from pathlib import Path
from typing import Any, Sequence

# 导入表情分析器
from .analyzer import EmotionAnalyzer

# 导入MCP相关模块
from mcp.server.models import InitializationOptions
import mcp.types as types
from mcp.server import NotificationOptions, Server
import mcp.server.stdio


# 全局变量
analyzer = None


async def main():
    """主函数"""
    global analyzer
    
    # 获取API密钥
    api_key = os.getenv("OPENROUTER_API_KEY")
    if not api_key:
        print("错误: 未设置OPENROUTER_API_KEY环境变量", file=sys.stderr)
        print("请设置环境变量: export OPENROUTER_API_KEY=your_api_key_here", file=sys.stderr)
        sys.exit(1)
    
    # 创建分析器
    analyzer = EmotionAnalyzer(api_key)
    
    # 创建服务器
    server = Server("emotion-analyzer")
    
    @server.list_tools()
    async def handle_list_tools() -> list[types.Tool]:
        """列出可用工具"""
        return [
            types.Tool(
                name="analyze_emotion",
                description="分析图像中人物的表情和情绪状态",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "image_path": {
                            "type": "string",
                            "description": "图像文件的绝对路径"
                        }
                    },
                    "required": ["image_path"]
                }
            ),
            types.Tool(
                name="analyze_emotion_base64",
                description="分析base64编码图像中人物的表情和情绪状态",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "image_data": {
                            "type": "string",
                            "description": "base64编码的图像数据"
                        },
                        "image_format": {
                            "type": "string",
                            "description": "图像格式（如：jpeg, png）",
                            "default": "jpeg"
                        }
                    },
                    "required": ["image_data"]
                }
            )
        ]
    
    @server.call_tool()
    async def handle_call_tool(
        name: str, arguments: dict[str, Any] | None
    ) -> list[types.TextContent | types.ImageContent | types.EmbeddedResource]:
        """处理工具调用"""
        if name == "analyze_emotion":
            return await _analyze_emotion_from_path(arguments)
        elif name == "analyze_emotion_base64":
            return await _analyze_emotion_from_base64(arguments)
        else:
            raise ValueError(f"未知工具: {name}")
    
    async def _analyze_emotion_from_path(arguments: dict[str, Any] | None) -> list[types.TextContent]:
        """从文件路径分析表情"""
        if not arguments:
            raise ValueError("缺少参数")
        
        image_path = arguments.get("image_path")
        if not image_path:
            raise ValueError("缺少图像路径参数")
        
        if not Path(image_path).exists():
            raise ValueError(f"图像文件不存在: {image_path}")
        
        try:
            result = analyzer.analyze_emotion(image_path)
            return [
                types.TextContent(
                    type="text",
                    text=json.dumps(result, ensure_ascii=False, indent=2)
                )
            ]
        except Exception as e:
            raise ValueError(f"分析错误: {str(e)}")
    
    async def _analyze_emotion_from_base64(arguments: dict[str, Any] | None) -> list[types.TextContent]:
        """从base64数据分析表情"""
        if not arguments:
            raise ValueError("缺少参数")
        
        image_data = arguments.get("image_data")
        image_format = arguments.get("image_format", "jpeg")
        
        if not image_data:
            raise ValueError("缺少图像数据参数")
        
        # 创建临时文件
        import tempfile
        import base64
        
        try:
            image_bytes = base64.b64decode(image_data)
        except Exception as e:
            raise ValueError(f"无效的base64数据: {str(e)}")
        
        # 创建临时文件
        with tempfile.NamedTemporaryFile(suffix=f".{image_format}", delete=False) as temp_file:
            temp_file.write(image_bytes)
            temp_path = temp_file.name
        
        try:
            result = analyzer.analyze_emotion(temp_path)
            result["image_path"] = f"临时文件 ({image_format}格式)"
            
            return [
                types.TextContent(
                    type="text",
                    text=json.dumps(result, ensure_ascii=False, indent=2)
                )
            ]
        except Exception as e:
            raise ValueError(f"分析错误: {str(e)}")
        finally:
            # 清理临时文件
            try:
                os.unlink(temp_path)
            except:
                pass
    
    # 运行服务器
    async with mcp.server.stdio.stdio_server() as (read_stream, write_stream):
        await server.run(
            read_stream,
            write_stream,
            InitializationOptions(
                server_name="emotion-analyzer",
                server_version="1.0.0",
                capabilities=server.get_capabilities(
                    notification_options=NotificationOptions(),
                    experimental_capabilities={},
                ),
            ),
        )


if __name__ == "__main__":
    asyncio.run(main())